from typing import List

from RsInstrument import *
from GFM313_Tx_tests.General.GFM313_utilities import *


class TxTestsBase:
    def __init__(self, resource_name_analyzer = None, resource_name_generator=None):
        try:
            if resource_name_analyzer is not None:
                self.analyzer = RsInstrument(resource_name_analyzer, True, True)
                self.analyzer.visa_timeout = 5000
            if resource_name_generator is not None:
                self.generator = RsInstrument(resource_name_generator, True, True)
                self.generator.visa_timeout = 5000
        except ResourceError as e:
            raise RsException(e.args[0])

    def instrument_settings(self, trigger_config, marker_config):
        """
        This function sets instrument related settings.
        :param trigger_config: Trigger configuration:  Armed Auto ('AAUT') or unchanged ('UNCH')
        :param marker_config: Marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')
        :return: Instrument status
        """
        if trigger_config is not None:
            self.generator.write_str(f':BB:NR5G:TCW:TRIG {trigger_config}')
        if marker_config is not None:
            self.generator.write_str(f':BB:NR5G:TCW:MARK {marker_config}')
        return self.generator.query_str(':syst:err?')

    def reset_analyzer(self):
        """
        This function resets the connected instrument
        :return: Instrument status
        """
        self.analyzer.write_str('*RST')
        return self.analyzer.query_str(':syst:err?')

    def reset_generator(self):
        """
        This function resets the connected interferer generator
        :return: Instrument status
        """
        self.generator.write_str('*RST')
        return self.generator.query_str(':syst:err?')

    def output_on(self, output: int):
        """
        This function switches the output of the  on
        :param output: Output number
        :return: Instrument status
        """
        self.generator.write_str(f':OUTPut{output}:STAT 1')
        return self.generator.query_str(':syst:err?')

    def output_off(self, output: int):
        """
        This function switches the output off
        :param output: Output number
        :return: Instrument status
        """
        self.generator.write_str(f':OUTPut{output}:STAT 0')
        return self.generator.query_str(':syst:err?')

    def close_analyzer(self):
        """
        This function closes the connection of the connected analyzer
        """
        self.analyzer.close()

    def close_generator(self):
        """
        This function closes the connection of the connected generator
        """
        self.generator.close()

    def query_status_analyzer(self):
        """
        This function queries the status of the connected analyzer
        :return: Instrument status
        """
        return self.analyzer.query_str(':syst:err?')

    def query_status_generator(self):
        """
        This function queries the status of the connected generator
        :return: Instrument status
        """
        return self.generator.query_str(':syst:err?')

    def query_errors_analyzer(self):
        """
        This function queries all errors of the connected analyzer
        :return: List of errors
        """
        return self.analyzer.query_all_errors()

    def query_errors_generator(self):
        """
        This function queries all errors of the connected generator
        :return: List of errors
        """
        return self.generator.query_all_errors()

    # ????
    def _check_device_suitability(self, needed: List[str]):
        """This function returns missing, required and already existing device options."""
        missing = []
        for option in needed:
            if option not in self.analyzer.instrument_options:
                missing.append(option)
        if len(missing) > 0:
            raise RsException(
                f'Some of the required options are missing.\nRequired: {", ".join(needed)}\n'
                f'Missing: {", ".join(missing)}\nAll options: {", ".join(self.analyzer.instrument_options)}')

    def hardcopy_analyzer(self, filename: str):
        """
        Save hardcopy of connected analyzer to project directory
        """
        self.analyzer.write_str('HCOP:DEV:LANG1 PNG')
        path = r'c:\R_S\Instr\user\hardcopy.png'
        self.analyzer.write_str("MMEM:NAME '" + path + "'")
        self.analyzer.write_str_with_opc('HCOP:IMM1')
        self.analyzer.read_file_from_instrument_to_pc(path, f'{filename}.png')
