from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC633(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()

        self.frequency = 1000
        """Set center frequency in MHz"""
        self.qam256supportet = True
        """Set to 'True' if BS supports 256QAM. Set to 'False' if BS does not support 256QAM"""
        self.channelBW = 10
        """Set channel bandwidth in MHz"""
        self.frequencyRange = 'FR1'
        """Select frequency range: 'FR1' or 'FR2'"""
        self.scs = 15
        """Set subcarrier spacing in kHz"""
        self.duplexing = 'FDD'
        """Select duplexing mode: 'FDD' or 'TDD'"""
        self.powerbackoff = True
        """Select power back off: 'True' or 'False'"""

    def start_measurement1(self):
        """
        This function starts the first measurement (OSTP1)
        :return:
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL NR5G')
            self.analyzer.write_str_with_opc('CONF:MEAS EVM')
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            # first measurement
            if self.qam256supportet:
                if self.powerbackoff:
                    # TM 3.1
                    testmodelstring = f"NR-{self.frequencyRange}-TM3_1__{self.duplexing}_{self.channelBW}MHz" \
                                      f"_{self.scs}kHz"
                    self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{testmodelstring}'")
                else:
                    testmodelstring = f"NR-{self.frequencyRange}-TM3_1a__{self.duplexing}_{self.channelBW}MHz" \
                                      f"_{self.scs}kHz"
                    self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{testmodelstring}'")
            else:
                # TM3.1
                testmodelstring = f"NR- {self.frequencyRange}-TM3_1__{self.duplexing}_{self.channelBW}MHz_{self.scs}kHz"
                self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{testmodelstring}'")
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')
            return self.analyzer.query_float('FETC:SUMM:OSTP?')
        except RsInstrException as e:
            raise RsException(e.args[0])

    def start_measurement2(self):
        """
        This function starts the second measurement (OSTP2)
        :return:
        """
        try:
            # second measurement
            if self.qam256supportet:
                # TM2a
                testmodelstring = f"NR-{self.frequencyRange}-TM2a__{self.duplexing}_{self.channelBW}MHz_{self.scs}kHz"
                self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{testmodelstring}'")
            else:
                # TM2
                testmodelstring = f"NR-{self.frequencyRange}-TM2__{self.duplexing}_{self.channelBW}MHz_{self.scs}kHz"
                self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{testmodelstring}'")
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')
            return self.analyzer.query_float('FETC:SUMM:OSTP?')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """This function returns the total dynamic range"""
        total_dynamic_range = self.start_measurement1() - self.start_measurement2()
        return total_dynamic_range

    def csv_export(self):
        """
        Exports the results summary to csv file (project directory).
        :return:
        """
        with open('results_633_TotalPowDynRange.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = [self.return_results()]
            header = ['Total power dynamic range [dBm]']
            entries = [header, results]
            writer.writerows(entries)
