from GFM313_Tx_tests.General.GFM313_base import TxTestsBase
from GFM313_Tx_tests.General.GFM313_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC64(TxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset_analyzer()

        self.frequency = 1000
        """Set center frequency in MHz"""
        self.testmodel = 'NR-FR1-TM1_1__TDD_15MHz_15kHz'
        """Set test model string"""
        self.noise_cancellation = True
        """Switch noise cancellation on/off: 'True' or 'False'"""
        self.power_unit = 'dBm'
        """Select power unit: 'dBm' or 'dBm/MHz'"""

    def start_measurement(self):
        """
        This function starts the measurement.
        """
        try:
            self.analyzer.write_str_with_opc('INST:SEL NR5G')
            self.analyzer.write_str_with_opc('CONF:MEAS TPOO')
            self.analyzer.write_float(':SENS:FREQ:CENT', self.frequency * 1000000)
            self.analyzer.write_str_with_opc(f"MMEM:LOAD:TMOD:CC1 '{self.testmodel}'")
            if self.noise_cancellation:
                self.analyzer.write_str_with_opc('NR5G:OOPower:NCOR 1')
            elif not self.noise_cancellation:
                self.analyzer.write_str_with_opc('NR5G:OOPower:NCOR 0')
            if self.power_unit == 'dBm':
                self.analyzer.write_str_with_opc('UNIT:OPOW DBM')
            elif self.power_unit == 'dBm/MHz':
                self.analyzer.write_str_with_opc('UNIT:OPOW DMHZ')

            self.analyzer.write_str(':INIT:CONT OFF')
            self.analyzer.write_str_with_opc('INIT:IMM')

        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function returns the transmit on/off power results summary: [Start OFF period limit (s), Stop OFF period
        limit (s), Time at delta to limit (s),OFF power (dBm or dBm/MHz), OFF power delta to limit (dB),
        Falling transmission period (s), Rising transmission period (s)]
        """
        try:
            data = self.analyzer.query_bin_or_ascii_float_list_with_opc('TRAC6:DATA? LIST')
        except RsInstrException as e:
            raise RsException(e.args[0])

        results = [data[0:7], data[7:14]]
        return results

    def csv_export(self):
        """
        Exports the results summary to csv file (project directory).
        """
        with open('results_64_TransmitOnOffPow.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = self.return_results()
            header = ['Start OFF period limit [s]', 'Stop OFF period limit [s]', 'Time at delta to limit [s]',
                      'OFF power' + self.power_unit, 'OFF power delta to limit [dB]', 'Falling transmission period [s]',
                      'Rising transmission period [s]']
            entries = [header, results[0], results[1]]
            writer.writerows(entries)
